%=========================================================================%
%                         RUNGE KUTTA                                     %
%%
%                  mx'' + cx' +kx = Q(t)                                  %
%                                                                         %
%                  x'(k+1) = x'(k) + h/6(a1 + 2a2 + 2a3 + a4)             %
%                  x(k+1)  = x(k) + h/6(b1 + 2b2 + 2b3 + b4)              %
%                                                                         %
%%
%  This program is written for the Master Thesis - 'Responce analysis of  %
%  a deep-draft dry tree semi-submersible' the spring 2012. It solves     %
%  a nonlinear equation of motion in the time domain.                     %
%                                                                         %
%                Authors: Marianne-Isabelle Falk                          %
%                         Thomas Skorpen                                  %
%%
%                         Oslo 10.june 2012                               %
%=========================================================================%
%                         DEFINING VARIABLES                              %
%%
% A33                   : Added mass coefficient in heave                 %
% b_pon                 : Bredth of pontoon                               %
% B33                   : Damping coefficient in heave                    %
% beta                  : Frequency ratio                                 %
% c_d_equviv            : Drag coefficient upscaled                       %
% c_d_1                 : This should be 2,5 based on DNV rules           %
% c_d_2                 : Approximation based on DNV rules                %
% d_eqviv               : Equvivalent diameter used for upscaled Cd       %
% d_pon                 : Hight of pontoon                                %
% depth                 : Depth from the free surface to the sea floor    %
% draught               : Draugth of semi                                 %
% draught_pon           : 'Middle' draugth of semi                        %
% dx                    : Velocity in heave                               %
% eta                   : Damping ratio                                   %
% F3                    : Exciting forces in heave                        %
% g                     : Acceleration of gravity                         %
% h                     : Timestep                                        %
% Hs                    : Significant wave height [m]                     %
% k_wave                : Wave number                                     %
% K33                   : Restoring term in heave                         %
% L/l_pon               : Length of the pontoons                          %
% lambda                : Wavelength                                      %
% mass                  : Mass of the semi-submersible                    %
% omega                 : Angular frequency [Hz]                          %
% Period                : Vector consisting of all periods                %
% phase_angle           : Force phase angle in degrees                    %
% rho                   : Density of sea water                            %
% T                     : Choosen period [s]                              %
% time                  : Length of time interval in [s]                  %
% theta                 : Phase angle                                     %
% vol                   : Displaced volume                                %
% x                     : Displacement in heave                           %
% z_wave_pon1(2)        : Velocity of wave at pontoon 1 (2)               %
%                                                                         %
%%
%      Variables from the hydropneumatic riser tensioner system           %
%%
% z_0                   : Air volume column relative to stroke length     %    
% n_g                   : Gas constant                                    %
% n_TTR                 : Number of TTRs                                  %
% zmax                  : Max-stroke                                      %
% risertension          : Pretension in each TTR in tonnes                %
% Totaltension          : Calculate total tension for system              %
% k_riser               : Spring rate of riser as N/m                     %
% k_TTR_lin             : Linearized spring stiffness                     %
% cn_tonnes             : Viscous damping coefficient                     %
% cn_lin                : Linear Viscous damping coefficient              %
% cn_nonlin             : Nonlinear viscous damping coefficient           %
% mu_col                : Coulomb friction as fraction of total tension   %
% lindamping            : Linearized damping                              %
% nonlindamping         : Nonlinear damping                               %
% coldamping            : Coulomb damping                                 %
% k_TTR                 : Riser tensioner stiffness                       %
% k_TTR_linearized      : Linearized value for k_TTR                      %
% naturalperiodlinearized : Linearized natural period                     %
%                                                                         %
%=========================================================================%

clear all;
clc;
hold on;

%-------------------------------------------------------------------------%
%               Data from the pre-project: RAO_TTR_preproject
%%
% fid = fopen('RAO_TTR_preproject.txt','r');
% RAO_TTR_preproject = fscanf(fid,'%f',[1 35]);
% fclose(fid);
%%
%               Uploading environmental and model data                    %
%%
%               with Morison damping included                             %
% fid = fopen('Runge_Kutta_input_from_HydroD.txt','r');                           
% temp1              = fscanf(fid,'%g %g %g %g %g %g %g',[7 1]);
% temp2              = fscanf(fid,'%f',[1 35]);  
% temp3              = fscanf(fid,'%f',[1 35]);      
% temp4              = fscanf(fid,'%f',[1 35]);
% temp5              = fscanf(fid,'%f',[1 35]);

%               without Morison damping included                          %
fid = fopen('RK_input_from_HydroD_wo_Morison.txt','r');                           
temp1              = fscanf(fid,'%g %g %g %g %g %g %g',[7 1]);
temp2              = fscanf(fid,'%f',[1 35]);  
temp3              = fscanf(fid,'%f',[1 35]);      
temp4              = fscanf(fid,'%f',[1 35]);
temp5              = fscanf(fid,'%f',[1 35]);
temp6              = fscanf(fid,'%f',[1 35]);
% phase_angle        = temp6(1,:); % for force in degrees!!!

%%
%                         ENVIRONMENTAL DATA & SEMI-SUB
%%
Period             = temp2(1,:);
Hs                 = 2;
h                  = 1;
time               = 400; t = [1:h:time];

%%
g                  = temp1(1);
rho                = temp1(2);
depth              = temp1(3);
vol                = temp1(5);
L                  = temp1(6);
draught            = 4.41960E+01;
d_pon              = 10.0584002;
draught_pon        = round(draught-d_pon/2);
l_pon              = round(79.7724978);
b_pon              = l_pon/2;
d_equiv            = 0.001;
c_d_equviv         = 56229;
c_d_1              = 2;
c_d_2              = 0.29;
%%
mass               = temp1(4);
A33_vector         = (rho*vol*temp3(1,:));
B33_vector         = (rho*vol*sqrt(g/L)*temp4(1,:));
K33                = temp1(7);
F3_vector          = rho*vol*g/L*temp5(1,:);
fclose(fid);
%%
%                          HYDROPNEUMATIC RISER TENSIONER SYSTEM          %
%%
z_0                = 31.5;
n_g                = 1.4;
n_TTR              = 12;
zmax               = 6.3;
risertension       = 840;
Totaltension       = risertension*1000*g*n_TTR;
k_riser            = 3600000;
k_TTR_lin          = 2.3591e+06;

cn_tonnes          = 1180;
cn_lin             = cn_tonnes*10000;
cn_nonlin          = 800*10000;
mu_col             = 0.02;


%-------------------------------------------------------------------------%
%                DEFINING VECTORS USED IN THE FOR-LOOP                    %
%-------------------------------------------------------------------------%
x                  = zeros(1,length(t));
dx                 = zeros(1,length(t));
z_wave_pon1        = zeros(1,length(t));
z_wave_pon2        = zeros(1,length(t));
c_mor1             = zeros(1,length(t));
c_mor2             = zeros(1,length(t));
c_mor3             = zeros(1,length(t));
morison            = zeros(1,length(t));
Tensionvariation   = zeros(1,length(t));
egenperiode        = zeros(1,length(i));
eigenfrequency     = zeros(1,length(t));
eta                = zeros(1,length(t));
beta               = zeros(1,length(t));
theta              = zeros(1,length(t));
nodes              = -b_pon:1:b_pon;
z_wave3            = zeros(length(t),length(nodes));

% The following variables are used to plot out data
Vector_beta        = zeros(1,length(Period));
Vector_theta       = zeros(1,length(Period));
Force              = zeros(1,length(Period));
force_max          = zeros(1,length(Period));
Morison_max        = zeros(1,length(Period));
Matrix_t_x         = zeros(length(Period),length(t));
Matrix_t_dx        = zeros(length(Period),length(t));
Matrix_t_c_mor2    = zeros(length(Period),length(t));
Matrix_t_c_mor3    = zeros(length(Period),length(t));
Matrix_t_morison   = zeros(length(Period),length(t));
Matrix_t_wave_pon1 = zeros(length(Period),length(t));
Matrix_t_wave_pon2 = zeros(length(Period),length(t));
Matrix_t_wave_pon3 = zeros(length(Period),length(t));
Matrix_t_theta     = zeros(length(Period),length(t));
Matrix_t_c         = zeros(length(Period),length(t));
Matrix_Tensionvariation                  = zeros(length(Period),length(t));
Matrix_Tensionvariation_tonnes           = zeros(length(Period),length(t));
Matrix_Tensionvariationlinearized_tonnes = zeros(length(Period),length(t));


for r = 1:length(Period)
    T       = Period(r)
    omega   = (2*pi/T);
    k_wave  =(omega^2)/g;
    

    %%
    %                          HYDRODYNAMIC COEFFICIENTS
    %%
    A33     = A33_vector(r);
    B33     = B33_vector(r);
    F3      = F3_vector(r)*sin(omega*t)*Hs/2;
    Wave    = (Hs/2)*sin((2*pi/13)*t);
    
    %%
    %                          TO BE USED FOR DECAY-SIMULASJON.
    %                          NOT OTHERWISE
    %                          for normal analyse, uncomment F3 above
    %%    
%     F3       = zeros(1,length(t));
%     F3(1:50) = 1000000;
    %%    
   
    disp('Solving the equation of motion with Runge-Kutta iteration ')
    for i = 1:h:length(t)
        
        z_wave_pon1(i) = omega*(Hs/2)*exp(-k_wave*draught_pon)*cos((omega*i)-(k_wave*b_pon)-theta(i));
        z_wave_pon2(i) = omega*(Hs/2)*exp(-k_wave*draught_pon)*cos((omega*i)-(k_wave*(-b_pon))-theta(i));
        
        if i < (length(t)-1)
        
            m_TTR      = (Totaltension*(1+(x(i)/z_0)).^n_g)/g;
            k_TTR      = Totaltension*n_g*((z_0.^(n_g))./((z_0-(x(i))).^(n_g+1)));
            c_mor1(i)  = 6.0431e+006*(z_wave_pon1(i)-dx(i)).*abs(z_wave_pon1(i)-dx(i));
            c_mor2(i)  = 6.0431e+006*(z_wave_pon2(i)-dx(i)).*abs(z_wave_pon2(i)-dx(i));
            for j=1:length(nodes)
                z_wave3(i,j) = omega*(Hs/2)*exp(-k_wave*draught_pon)*cos((omega*t(i))-(k_wave*(nodes(j)))-theta(i));
                sumwave3     = sum(z_wave3')/length(nodes);
                c_mor3(i)    = 2*6.0431e+006*(sumwave3(i)-dx(i)).*abs(sumwave3(i)-dx(i));
            end
            morison(i) = c_mor1(i)+c_mor2(i)+c_mor3(i);
           
%             morison_166 = 8.515E+06;
%             morison_1   = 5.1294e+006;
            Matrix_t_c_mor3(r,i)    = c_mor3(i);
            Matrix_t_c_mor2(r,i)    = c_mor2(i);
            Matrix_t_morison(r,i)   = morison(i);
            Matrix_t_wave_pon1(r,i) = z_wave_pon1(i)-dx(i);
            Matrix_t_wave_pon2(r,i) = z_wave_pon2(i)-dx(i);
            Matrix_t_wave_pon3(r,i) = sumwave3(i)-dx(i);
           
                    
            m = mass + A33;
            c = B33 + cn_lin;
            k = K33 + k_TTR;
            Q = F3 + morison(i) - Totaltension*mu_col*sign(dx(i));
            
            eigenfrequency(i)   = (sqrt(k/m));
            beta(i)             = omega/eigenfrequency(i);
            eta(i)              = c/(2*m*eigenfrequency(i));
            theta(i)            = atan((2*eta(i)*beta(i))/(1-(beta(i))^2));
            if theta(i) < 0
                theta(i) = pi + theta(i);
            end
            Matrix_t_theta(r,i) = theta(i);
            Matrix_t_c(r,i)     = c;
                       
            a1 = 1/m*(Q(i) - k*x(i) - c*dx(i));
            b1 = dx(i);
            a2 = 1/m*(((Q(i)+Q(i+1))/2) - k*(x(i)+0.5*h*b1) - c*(dx(i)+0.5*h*a1));
            b2 = dx(i) + 0.5*h*a1;
            a3 = 1/m*(((Q(i)+Q(i+1))/2) - k*(x(i)+0.5*h*b2) - c*(dx(i)+0.5*h*a2));
            b3 = dx(i) + 0.5*h*a2;
            a4 = 1/m*(Q(i+1) - k*(x(i)+h*b3) - c*(dx(i)+h*a3));
            b4 = dx(i) + h*a3;
            
            dx(i+1)             = dx(i) + h/6*(a1 + 2*a2 + 2*a3 + a4);
            x(i+1)              = x(i) + h/6*(b1 + 2*b2 + 2*b3 + b4);

Matrix_Tensionvariation(r,i) = Totaltension+k_TTR*x(i)+cn_lin*dx(i)+Totaltension*mu_col*sign(dx(i));
Matrix_Tensionvariation_tonnes(r,i) = (Totaltension+k_TTR*x(i)+(cn_lin*dx(i))+Totaltension*mu_col*sign(dx(i)))/10000;
Matrix_Tensionvariationlinearized_tonnes(r,i) = (Totaltension+k_TTR_lin*x(i)+(cn_lin*dx(i))+Totaltension*mu_col*sign(dx(i)))/10000;

        end
%        egenperiode(r,i)         = 2*pi/(sqrt(k/m));
        Matrix_t_x(r,i)         = x(i);
        Matrix_t_dx(r,i)        = dx(i);
        Force(r,i)              = Q(i);
    end
end

disp('Time iteration finished')
disp(' ')
disp(' ')
disp(' ')


disp('Calculating the RAO in the frequency domain');
%-------------------------------------------------------------------------%
%    CALCULATES THE RAO & OTHER VARIABLES THAT VARIES OVER THE PERIOD     %
%-------------------------------------------------------------------------%
x_max  = zeros(1,length(Period));
dx_max = zeros(1,length(Period));

for i = 1:length(Period)
    x_max(i)       = max(Matrix_t_x(i,(time-100):(length(t)/h)));
    dx_max(i)      = max(Matrix_t_dx(i,(time-100):(length(t)/h)));
    force_max(i)   = max(Force(i,(time-100):(length(t)/h)));
    Morison_max(i) = max(Matrix_t_morison(i,(time-100):(length(t)/h)));
end



disp('Calculating the linear and nonlinear damping and K_TTR')
%-------------------------------------------------------------------------%
%           CALCULATES LINEAR AND NONLINEAR DAMPING AND K_TTR             %
%-------------------------------------------------------------------------%
lindamping         = cn_lin.*dx;
nonlindamping      = cn_lin.*dx.*abs(dx);
coldamping         = Totaltension*mu_col*sign(dx(i));
k_TTR              = Totaltension*n_g*((z_0.^(n_g))./((z_0-(x)).^(n_g+1)));

% Using the last period = 40s to calculate the spring rate 
k_TTR_linearized   = sum(k_TTR(310:349))/40;
naturalperiodlinearized = (2*pi)*sqrt((mass+A33)/(K33+k_TTR_linearized));



disp(' ')
disp(' ')
disp(' ')
disp('Analysis finished :)')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%