%=========================================================================%
%                         TENSIONER DAMPING                               %
%%
%  This program is written for the Master Thesis - 'Responce analysis of  %
%  a deep-draft dry tree semi-submersible' the spring 2012. It calculates %
%  the pressure loss in an oscillating accumulator based on the Bernoulli %
%  equation and conservation of mass in the gas and fluid.                %
%                                                                         %
%                Authors: Marianne-Isabelle Falk                          %
%                         Thomas Skorpen                                  %
%%
%                         Oslo 15.june 2012                               %
%                                                                         %
%=========================================================================%
%                         DEFINING VARIABLES                              %
%%
%      Variables when calculating loss in each cylinder                   %
%%
% A_cyl                 : Area of piston in m^2                           %
% A_cyl2                : Real piston area used for calculating tension   %
% amp                   : Amplitude                                       %
% omega                 : Oscillating frequency                           %
% p_oil                 : Density of oil in kg/m^3                        %
% Period                : Oscillating period                              %
% press_0               : Initial pressure in system in N/m^3 (Pa)        %
% Q_oil                 : Oil flow m^3/s                                  %
% resp                  : Response as function of time                    %
% t                     : Time interval                                   %
% v_oil                 : Viscosity of hydraulic oil in CST               %
% vel                   : Velocity as function of time                    %
% vol_acc               : Volume of accumulator in m^3                    %
% vol_gas               : Volume in gas bottles in m^3                    %
%                                                                         %
%%
%      Variables when calculating loss in piping/bends for hydraulics     %
%%
% A_1                   : Area of piston in m^2                           %
% D_1                   : Diameter of piping in meters                    %
% E_1                   : Surface roughness in m                          %
% L_1                   : Length of piping in meters                      %
% plossbar_1            : Pressure loss given in bar                      %
% rey_1                 : Reynolds number                                 %
% vel_1                 : Velocity in piping as function of time          %    
%                                                                         %
%%
%      Variables when calculating loss in piping/bends for pneumatics     %
%%
% A_g1                  : Inside area of piping #1                        %
% D_g1                  : Diameter of piping #1                           %
% dgasvol_acc           : Change of gas volume                            %
% gasvol_acc            : Gas volume in accumulator                       %
% gfriction_1           : Friction factor                                 %
% L_g1                  : Length of piping #1                             %
% p0_g1                 : Density of gas                                  %
%                                                                         %
%%
%      Variables when calculating the Tension variation                   %
%%
% tension               : Tension in each cylinder                        %
% tensiontotal          : Total tension in the system                     %
%                                                                         %
%=========================================================================%

clear all;
clc;

%-------------------------------------------------------------------------%
%                  CALCULATING LOSS IN EACH CYLINDER                      %
%-------------------------------------------------------------------------%

% Input for oscillation of wave / response
period           = 14;
omega            = (2*pi)/period;
t                = 1:1:100;
amp              = 0.01;
resp             = amp*sin(omega*t);
vel              = amp*omega*cos(omega*t);

% Input for accumulator and hydraulic oil
% Common viscosity of hydraulic oils is in the range 16 - 100 centistokes.
% Optimum viscosity value is 16 - 36 centistokes.
vol_acc          = 6;
vol_gas          = 20;
A_cyl            = 4*0.25*pi*(0.56^2);
A_cyl2           = 0.25*pi*(0.56^2);
v_oil            = 30*(10^-6);
p_oil            = 872;
press_0          = 91.34*(10^5);

% Flow of oil between cylinder and accumulator
Q_oil            = A_cyl*vel;




%-------------------------------------------------------------------------%
%       CALCULATING THE PRESSURE LOSS IN PIPING/BENDS FOR HYDRAULICS      %
%-------------------------------------------------------------------------%

% Component 1
D_1              = 0.2;
A_1              = 0.25*pi*D_1^2;
L_1              = 30;
E_1              = 0.0002;
vel_1            = Q_oil/(A_1);
rey_1            = (vel_1*D_1)/(v_oil);

%%
%                  Different friction models                              %
%%
% Haaland equation for darcys friction
friction_1       = (1./(-1.8*log10((E_1./(D_1*3.7))^1.11+(6.9./rey_1)))).^2;

% Swameee-Jaim implicit darcys friction equation
sj_friction_1    = 0.25./(log10((E_1./(3.7.*D_1))+(5.74./rey_1.^2))).^2;

% Brkic solution for darcys friction factor
brkic_s1         = log(rey_1/(1.816*log((1.1*rey_1)/(log(1+(1.1*rey_1))))));
brkic_friction_1 = (1./(-2*log10((E_1./(D_1*3.71))+((2.18*brkic_s1)./rey_1)))).^2;
%%

ploss_temp       = 0.5.*p_oil.*((friction_1.*L_1)./D_1).*vel_1.*abs(vel_1);
ploss_temp2      = ploss_temp/4;
ploss_1          = ploss_temp2*2;
plossbar_1       = ploss_1/(10^5);




%-------------------------------------------------------------------------%
%       CALCULATING THE PRESSURE LOSS IN PIPING/BENDS FOR PNEUMATICS      %
%-------------------------------------------------------------------------%

p0_g1            = 1.24;
D_g1             = 0.2;
L_g1             = 20;
A_g1             = 0.25*pi*D_g1^2;

gasvol_acc       = (vol_acc/2)+(resp*A_cyl);
dgasvol_acc      = vel*A_cyl;

gfriction_1      = 1;



%-------------------------------------------------------------------------%
%                   CALCULATING THE TENSION VARIATION                     %
%-------------------------------------------------------------------------%

tension          = 4*(((press_0)+ploss_1)*A_cyl2);
tensiontonnes    = tension/10000;
tensiontotal     = 12*tensiontonnes;


%-------------------------------------------------------------------------%
%                   PLOTTING THE TENSION VARIATION                        %
%-------------------------------------------------------------------------%

hold on;
figure(1)
plot(resp,tensiontotal)
title('Tension variation in top tensioned risers')
ylabel('Tension in risers [tonnes]')
xlabel('Displacement [m]') 

figure(2)
plot(vel,tensiontotal)
title('Tension variation in top tensioned risers')
ylabel('Tension in risers [tonnes]')
xlabel('Velocity [m/s]') 

%=========================================================================%